@@ -287,6 +287,7 @@ class Metric():
 
     def load_metric_from_json(self, metric_json, verbose = True, **params):
 
+        self.json = metric_json
         load = metric_json
 
         self.name = load['name']
@@ -295,6 +296,7 @@ class Metric():
             print("Loading {}".format(self.name))
         
         self.x = []
+        self.dx = []
         self.x_str = []
         self.u = []
 
@@ -308,6 +310,10 @@ class Metric():
             velocity = "u_" + coordinate
             setattr(self, velocity, sp.symbols(velocity))
             self.u.append(self.__dict__[velocity])
+
+            differential = "d" + coordinate
+            setattr(self, differential, sp.symbols(differential))
+            self.dx.append(self.__dict__[differential])
         
         self.g = sp.zeros(4, 4)
         self.g_inv = sp.zeros(4, 4)
@@ -342,13 +348,6 @@ class Metric():
         self.transform_functions = []
         self.transform_s = []
 
-        if "transform" in load:
-            for i in range(3):
-                transf = load['transform'][i]
-                transform_function = sp.parse_expr(transf)
-                self.transform_functions.append(sp.lambdify([self.x[1], self.x[2], self.x[3]], self.evaluate_constants(transform_function)))
-
-
         free_sym = list(self.g.free_symbols-set(self.x))
         
         self.initialized = 1
@@ -367,6 +366,13 @@ class Metric():
 
         self.initialized_metric = 1
         
+        if "transform" in load:
+            for i in range(4):
+                transf = load['transform'][i]
+                transform_function = sp.parse_expr(transf)
+                self.transform_s.append(transform_function)
+                self.transform_functions.append(sp.lambdify([self.x], self.evaluate_constants(transform_function), 'numpy'))
+
         if verbose:
             print("The metric_engine has been initialized.")
 
@@ -374,6 +380,7 @@ class Metric():
         if self.initialized:
             self.constants[symbol] = {}
             self.constants[symbol]['symbol'] = symbol
+            self.constants[symbol]['symbolic'] = sp.symbols(symbol)
             self.constants[symbol]['value'] = None
         else:
             print("Inizialize (initialize_metric) or load (load_metric) a metric before adding constants.")
@@ -398,6 +405,12 @@ class Metric():
         else:
             print("Inizialize (initialize_metric) or load (load_metric) a metric before adding constants.")
 
+    def get_constants_symb(self):
+        return [self.constants[constant]['symbolic'] for constant in self.constants]
+
+    def get_constants_val(self):
+        return [self.constants[constant]['value'] for constant in self.constants]
+
     def evaluate_constants(self, expr):
         if any(x['value'] == None for x in self.constants.values()):
             print("You must set_constant for every constant value in your metric, before evaluating.")
